#ifndef CITY_H
#define CITY_H

#include <vector>
#include <map>
#include <utility>



#include "road.h"
#include "building.h"
#include "mesh.h"
#include "vector.hpp"
#include "quadrangle.h"
#include "polygone.h"
#include "utility.h"

class City {
public:
    City();
    ~City();
    int init(Mesh&);

    inline Quadrangle const & getLand() const {
        return land;
    }

    inline std::vector<Polygone> const & getSurfaces() const {
        return surfaces;
    }

    inline std::vector<Building> const & getBuildings() const {
        return buildings;
    }

    void addBuilding(const Polygone&, Mesh&);

    Polygone raiseSurface(const vector3D&, const Polygone&,  Mesh&, std::string="default");
    
    void splitSurface(const Polygone&, Mesh&,int=0,int=0);

    void splitSurfaceGarden(const Polygone& ,const Polygone& ,int , Mesh& );    
    
    int checkSurface(const Polygone&);

    void createMesh(Mesh&);


protected:
private:
    Quadrangle land;
    std::vector<Polygone> surfaces;
    std::vector<Building> buildings;
    vector3D cityCenter;
};


#endif //CITY_H
