#ifndef MESH_H
#define MESH_H

#include <vector>
#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>

#include "vector.hpp"
#include "indexMesh.h"


class Mesh {
protected:
    std::vector<vector3D> points; //points
    std::vector<IndexMesh> indices; //indices

public:

    Mesh() {

    }

    ~Mesh() {

    }
    Mesh(const Mesh&);
    
    inline std::vector<vector3D> const & getPoints() const {
        return points;
    }
    
    inline std::vector<IndexMesh> const & getIndices() const {
        return indices;
    }
    
    void write(std::string const &) const;
    
    void addQuad(const vector3D&, const vector3D&, const vector3D&, const vector3D&, std::string = "default");
    void addTriangle(const vector3D&, const vector3D&, const vector3D&, std::string = "default");
    void addBox(const vector3D&, const vector3D&);
    void addBox3(const vector3D& , const vector3D& , const vector3D& , double = 0, std::string = "default");
    void addBox(const vector3D&, const vector3D&,const vector3D&, const vector3D&, double = 0, std::string = "default");
    void addBoxFull(const vector3D&, const vector3D&,const vector3D&, const vector3D&, double = 0, std::string = "default");
    void addPyramid(const vector3D&, const vector3D&, const vector3D&, const vector3D&, const vector3D&, std::string = "default");
    void addCylinder(const vector3D&, const vector3D&, double, int = 20, std::string = "default");
    void addCone(const vector3D&, const vector3D&, double, int = 20, std::string = "default");
    void addSphere(const vector3D&, double = 1.0, int = 20, int = 20, std::string = "default");

    //+ toutes les autres primitives....

    // static Mesh createBox();
    //friend Mesh merge(Mesh,Mesh);    
    //static Mesh createCylinder();
    
    friend std::ostream & operator <<(std::ostream &, const Mesh &);
private:



};

#endif //MESH_H
