#ifndef POLYGONE_H
#define POLYGONE_H

#include <vector>
#include "vector.hpp"
#include "line.h"
#include "triangle.h"
#include "quadrangle.h"
#include "mesh.h" // TODO <= remove
#include "utility.h"

class Triangle;
class Quadrangle;

class Polygone {
public:

    Polygone() {
    }

    ~Polygone() {
    }
    Polygone(const Polygone&);
    Polygone(const Triangle&);
    Polygone(const Quadrangle&);


    inline vector3D const & operator[](int i) const {
        return points[i];
    }

    inline vector3D & operator[](int i) {
        return points[i];
    }

    inline const std::vector<vector3D> getPoints() const {
        return points;
    }

    inline const int size() const {
        return points.size();
    }

    inline void push_back(const vector3D& p) {
        points.push_back(p);
    }

    Polygone& operator+=(const vector3D&);
    Polygone& operator-=(const vector3D&);

    /**
     * Calcul d'aire necessaire pour ne pas construire sur les surface trop petite apres réduction, à retirer si contrôle fait dans grammaire.
     */
    //TODO, merger les 2 fonctions
    inline double area() const {
        if (points.size() == 4) {
            return 0.5 * Norm((points[0] - points[1]) / (points[0] - points[2])) + 0.5 * Norm((points[0] - points[2]) / (points[0] - points[3]));
        } else if (points.size() == 3) {
            return 0.5 * Norm((points[0] - points[1]) / (points[0] - points[2]));
        }
        return 0;
    }
    
    inline double area(const std::vector<vector3D>& pts) const {
        if (pts.size() == 4) {
            return 0.5 * Norm((pts[0] - pts[1]) / (pts[0] - pts[2])) + 0.5 * Norm((pts[0] - pts[2]) / (pts[0] - pts[3]));
        } else if (pts.size() == 3) {
            return 0.5 * Norm((pts[0] - pts[1]) / (pts[0] - pts[2]));
        }
        return 0;
    }
    
    inline vector3D getCenter() const {
        int centerX = 0, centerY = 0, centerZ = 0;
        for (std::vector<vector3D>::const_iterator it = points.begin(); it != points.end(); it++) {
            centerX += (*it)[0];
            centerY += (*it)[1];
            centerZ += (*it)[2];
        }
        return vector3D(centerX / points.size(), centerY / points.size(), centerZ / points.size());
    }

    int reduce(int, int = -1);

    friend std::ostream & operator <<(std::ostream &, const Polygone &);

protected:
    std::vector<vector3D> points;
private:
};

#endif //POLYGONE_H
