#ifndef QUADRANGLE_H
#define QUADRANGLE_H

#include <iostream>
#include <vector>
#include <cmath>
#include <ctime>  
#include <cstdlib>
#include "vector.hpp"
#include "polygone.h"
#include "triangle.h"
#include "utility.h"

class Polygone;

class Quadrangle {
public:
    Quadrangle();
    ~Quadrangle();
    Quadrangle(const Quadrangle&);
    Quadrangle(const vector3D&, const vector3D&, const vector3D&, const vector3D&);
    Quadrangle(const vector3D&, const vector3D&);

    inline vector3D const & operator[](int i) const {
        return points[i];
    }

    inline vector3D & operator[](int i) {
        return points[i];
    }

    inline vector3D const * const getPoints() const {
        return points;
    }

    /**
     * On suppose que le quad sera toujours convexe : pour faire simple prendre l'aire de deux triangles selon une diagonale
     */
    inline double area() const {
        return 0.5 * Norm((points[0] - points[1]) / (points[0] - points[2])) + 0.5 * Norm((points[0] - points[2]) / (points[0] - points[3]));
    }

    void grammaire(std::vector<Polygone>&) const;

    friend std::ostream & operator <<(std::ostream &, const Quadrangle &);

protected:

private:
    vector3D points[4];
    
    inline int getMaxEdgeRank(int dist01, int dist12, int dist23, int dist30) const {
        if (dist01 < dist12) {
            if (dist12 < dist23) {
                if (dist23 < dist30) {
                    return 3; //cote 3-0
                } else {
                    return 2; //cote 2-3
                }
            } else {
                if (dist12 < dist30) {
                    return 3; //cote 3-0
                } else {
                    return 1; //cote 1-2
                }
                
            }
        } else {
            if (dist01 < dist23) {
                if (dist23 < dist30) {
                    return 3; //cote 3-0
                } else {
                    return 2; //cote 2-3
                }
            } else {
                if (dist01 < dist30) {
                    return 3; //cote 2-3
                } else {
                    return 0; //cote 0-1
                }
            }
        }
    }
};

#endif //QUADRANGLE_H
