#ifndef UTILITY_H
#define UTILITY_H

#include <cmath>
#include <cstdio>
//City
#define WIDTH_CITY 500
#define LENGTH_CITY 500

inline double getCityDiagonal() {
    return sqrt(pow(WIDTH_CITY, 2) + pow(LENGTH_CITY, 2)); //City diagonal, don't touch this formula !
}

//Buildings
#define MIN_WIDTH_BUILDING 20
#define MAX_WIDTH_BUILDING 50
#define MIN_BUILDING_QUAD_DIAGONAL 30
#define MAX_BUILDING_QUAD_DIAGONAL 200
#define MIN_BUILDING_QUAD_ANGLE 20
#define MAX_BUILDING_QUAD_ANGLE 160
#define MIN_BUILDING_TRI_ANGLE 20
#define MAX_BUILDING_TRI_ANGLE 140
#define MIN_BUILDING_TRI_HEIGHT 30
#define MAX_BUILDING_TRI_HEIGHT 100
#define SUBURBAN_DIST_RATIO 0.6 //If a building is far from (city_diagonal * SUBURBAN_DIST_RATIO), this building is in the suburban area
#define CBD_DIST_RATIO 0.1 //If a building is close to (city_diagonal * CBD_DIST_RATIO), this building is in the Central Business District
//Others building are in the "normal" city area
#define BUILDING_ELEVATION_POWER 0.9 //The smaller the value is, the taller the buildings are. Highly recommended to be < 1, but not too small

//Grammar probability, in %. Total must be equal to 100, and substract the preceding to know the current
#define F_F_SUBURBAN 55 //Floor suburban probability
#define F_F_CBD 95 //Floor central business district probability
#define F_R 100 //Roof probability
#define F_F_SF 10 //Probability of surface reduction during elevation

//Floors
#define MIN_FLOORS_BUILDING 0 //A ground floor, and a roof
#define MAX_FLOORS_BUILDING 150 //X floors + 1 groud floor + 1 roof
#define MIN_HEIGHT_GROUNDFLOOR 4
#define MAX_HEIGHT_GROUNDFLOOR 4
#define MIN_HEIGHT_FLOOR 3
#define MAX_HEIGHT_FLOOR 3
#define MIN_HEIGHT_ROOF 0
#define MAX_HEIGHT_ROOF 3

//Roof
#define HIGH_COUNT_LEVEL 10
#define MID_COUNT_LEVEL 6
#define LOW_COUNT_LEVEL 1

//Windows and balcons
#define GENERATE_WINDOW_BALCON 1 // 0|1
#define WIDTH_WINDOW 1.5
#define HEIGHT_WINDOW 1.5
#define WIDTH_BORDER_WINDOW 0.1
#define WIDTH_BALCON 1 
#define LENGTH_BALCON 4
#define HEIGHT_BALCON 1
#define WIDTH_BORDER_BALCON 0.1

//Roads
#define WIDTH_ROAD_WAY 5
#define MIN_WIDTH_ROAD 10
#define MAX_WIDTH_ROAD 30
#define MIN_WIDTH_SIDEWALK 1
#define MAX_WIDTH_SIDEWALK 3
#define SIDEWALK_SURELEVATION 0.3

//Garden
#define MIN_WIDTH_GARDEN 10
#define MAX_WIDTH_GARDEN 1000
#define MIN_RATIO_BUILDING_GARDEN 20
#define MAX_RATIO_BUILDING_GARDEN 80
#define GARDEN_SURELEVATION 0.75

/* ----- District division ----- */

//Triangles
#define TRIANGLE_EDGE_MIN 25
#define TRIANGLE_SURF_MIN 1250 //(50 * 50) / 2
#define TRIANGLE_SURF_MAX 45000 //(300 * 300) / 2

//Quadrangles
#define QUADRANGLE_SURF_MIN 1225 //35 * 35
#define QUADRANGLE_SURF_MAX 40000 //200 * 200

//Probability of each division, in %. Total must be equal to 100, and substract the preceding to know the current
//Q divisions
#define Q_QQ 80
#define Q_QT 95
#define Q_TT 100
//Case of no Q_TT division
#define Q_QQ_NO_TT 90
#define Q_QT_NO_TT 100
//T divisions
#define T_TT 15
#define T_QT 100

#define STOP_DIVISION_POWER 2 //The higher the value is, the smaller areas are. But if it's to high, it can restrict variance in areas surface...
#define STOP_DIVISION_RANDOM 1000000 //Just for precision

/**
 * Function to stop - or not - triangle/quadrangle division in grammar function
 * @param min Minimum authorized area
 * @param max Maximum authorized area
 * @param surface Current area value
 * @return true if division must stop, false otherwise
 */
inline bool stopDivision(double min, double max, double surface) {
    //return 100.0 * exp(-5.0 * (surface -2.0 * min) / max) > rand() % (100 + 1);
    double diff = (max - min / 2);
    double down = max - surface;
    return (surface < min * 2) || (surface < max) && (pow((down / diff), pow(diff / down, STOP_DIVISION_POWER)) * STOP_DIVISION_RANDOM > rand() % (STOP_DIVISION_RANDOM + 1));
}

//Probability of each edge cutting ratio, in %. Total must be equal to 100, and substract the preceding to know the current
#define PRCT_04 10
#define PRCT_06 20
#define PRCT_05 100

/**
 * Generate a random edge cutting ratio. It permits to not necessarily cut a surface by 2 strict equals areas
 * @return the edge cutting ratio
 */
inline double getEdgeCutRatio() {
    int random = rand() % (100 + 1); //[1, 100]
    if (random < PRCT_04) {
        return 0.4;
    } else if (random < PRCT_06) {
        return 0.6;
    } else {
        return 0.5;
    }
}

#endif
