#include "building.h"
#include "polygone.h"

Building::Building(const Polygone & polygone) : surface(polygone) {
  Building::type = rand()%4;// Probablement a changer
}

void Building::grammaire(const vector3D & cityCenter) {
    int level = floors.size();
    const Polygone & previousSurface = floors[level - 1].getSurface();
    if (level <= MIN_FLOORS_BUILDING) { //Need a ground-floor
        floors.push_back(Floor(surface, 0, level));
        grammaire(cityCenter);
    } else if (level >= MAX_FLOORS_BUILDING) { //Stop elevation
        roof = Roof(previousSurface, type, level);
    } else {
        int f_f_prob = 0;
        double distSurfaceCityCenter = Norm(surface.getCenter() - cityCenter); //Distance from current surface to cityCenter
        if (distSurfaceCityCenter > getCityDiagonal() * SUBURBAN_DIST_RATIO) {
            f_f_prob = F_F_SUBURBAN;
        } else if (distSurfaceCityCenter < getCityDiagonal() * CBD_DIST_RATIO) {
            f_f_prob = F_F_CBD;
        } else {
            double diff = getCityDiagonal() * (SUBURBAN_DIST_RATIO - CBD_DIST_RATIO);
            double down = getCityDiagonal() * SUBURBAN_DIST_RATIO - distSurfaceCityCenter;
            f_f_prob = F_F_SUBURBAN + abs(F_F_SUBURBAN - F_F_CBD) * pow(down / diff, pow(diff/down, BUILDING_ELEVATION_POWER));
            //std::cout << distSurfaceCityCenter << " / " << diff << " / " << down << " / " << f_f_prob << std::endl;;
        }
        int randGrammar = rand() % 100;
        if (randGrammar < f_f_prob) { //New floor
            int randReduce = rand() % 100;
            if (randReduce < F_F_SF) { //Elevation with surface reduction
                floors.push_back(Floor(previousSurface, type, level)); //TODO reduce surface
            } else { //Normal elevation
                floors.push_back(Floor(previousSurface, type, level));
            }
            grammaire(cityCenter);
        } else { //End of building, make the roof
            roof = Roof(previousSurface, type, level);
        }
    }
}

void Building::createMesh(Mesh & m) const {
    for (std::vector<Floor>::const_iterator it = floors.begin(); it != floors.end(); it++) { //Dessiner le rez-de chaussé et les étages
        it->createMesh(m);
    }
    if (roof.getSurface().size() > 0) {
        roof.createMesh(m);
    }
}

std::ostream & operator <<(std::ostream & os, const Building & building) {
    os << "Surface: " << building.surface << std::endl;
    os << "Floors: " << std::endl;
    for (std::vector<Floor>::const_iterator it = building.floors.begin(); it != building.floors.end(); it++) {
        os << *it << std::endl;
    }
    os << "Roof: " << building.roof << std::endl;
}
