#include "floor.h"

Floor::Floor(const Polygone& surface, int type, int level) : surface(surface), type(type), level(level) {

}

void Floor::createMesh(Mesh &m) const {
  int elevation = (level == 0) ? 0 : MIN_HEIGHT_GROUNDFLOOR + MIN_HEIGHT_FLOOR * (level - 1);
  vector3D a(surface[0][0], surface[0][1], surface[0][2] + elevation);
  vector3D b(surface[1][0], surface[1][1], surface[1][2] + elevation);
  vector3D c(surface[2][0], surface[2][1], surface[2][2] + elevation);
  if (surface.getPoints().size() == 3) {
    m.addBox3(a, b, c, (level == 0) ? MIN_HEIGHT_GROUNDFLOOR : MIN_HEIGHT_FLOOR);
  } else if (surface.getPoints().size() == 4) {
    vector3D d(surface[3][0], surface[3][1], surface[3][2] + elevation);
    m.addBox(a, b, c, d, (level == 0) ? MIN_HEIGHT_GROUNDFLOOR : MIN_HEIGHT_FLOOR);
  }
  if(GENERATE_WINDOW_BALCON==1){
    addWindows(m, elevation);
    if(level > 0 and type%2==0 ) addBalcon(m, elevation);
  }
}

std::ostream & operator <<(std::ostream & os, const Floor & floor) {
  return os << "floor(surface: " << floor.surface << ", type: " << floor.type << ", level: " << floor.level << ")" << std::endl;
}

//Afficher aussi à l'interieur car apparement on se retrouve avec des polygones dans le sens direct et indirect...
void Floor::addBalcon(Mesh& m, int elevation) const{
  for(int i = 0 ; i<surface.size();i++){
    double length = Norm(surface[i+1==surface.size()?0:i+1]-surface[i]);
    double lengthBalcon = LENGTH_BALCON*(1+(type/10)*2);
    double widthBalcon = WIDTH_BALCON*(1+type/10);	
    int divs = length/lengthBalcon;
    double division = length / divs;
    vector3D dir = Normalized(surface[i+1==surface.size()?0:i+1]-surface[i]);
    vector3D vert = vector3D(-1 * dir[1],dir[0],dir[2]);
    //placer une fenetre toutes les 3 divisions
    for(int j = 2 ; j<divs; j+=4){
      {//bas
	vector3D a = (surface[i] + dir * (j*division) + vert * widthBalcon) + vector3D(0,0,elevation);
	vector3D b = (surface[i] + dir * (j*division + lengthBalcon) + vert * widthBalcon) + vector3D(0,0,elevation);
	vector3D c = (surface[i] + dir * (j*division + lengthBalcon) - vert * widthBalcon ) + vector3D(0,0,elevation);
	vector3D d = (surface[i] + dir * (j*division) - vert * widthBalcon) + vector3D(0,0,elevation);
	m.addBoxFull(a, b, c, d, WIDTH_BORDER_BALCON);
      }
      {//côté 1
	vector3D a = (surface[i] + dir * (j*division ) + vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D b = (surface[i] + dir * (j*division +WIDTH_BORDER_BALCON ) + vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D c = (surface[i] + dir * (j*division +WIDTH_BORDER_BALCON) - vert * widthBalcon ) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D d = (surface[i] + dir * (j*division) - vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	m.addBoxFull(a, b, c, d, HEIGHT_BALCON-WIDTH_BORDER_BALCON);
      }
      {//côté 2
	vector3D a = (surface[i] + dir * (j*division + lengthBalcon ) + vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D b = (surface[i] + dir * (j*division + lengthBalcon - WIDTH_BORDER_BALCON ) + vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D c = (surface[i] + dir * (j*division + lengthBalcon - WIDTH_BORDER_BALCON) - vert * widthBalcon ) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D d = (surface[i] + dir * (j*division + lengthBalcon) - vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	m.addBoxFull(a, b, c, d, HEIGHT_BALCON-WIDTH_BORDER_BALCON);
      }
      {//face1
	vector3D a = (surface[i] + dir * (j*division) + vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D b = (surface[i] + dir * (j*division + lengthBalcon) + vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D c = (surface[i] + dir * (j*division + lengthBalcon) + vert * (widthBalcon-WIDTH_BORDER_BALCON) ) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D d = (surface[i] + dir * (j*division) + vert * (widthBalcon-WIDTH_BORDER_BALCON)) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	m.addBoxFull(a, b, c, d, HEIGHT_BALCON - WIDTH_BORDER_BALCON);
      }
      {//face2
	vector3D a = (surface[i] + dir * (j*division) - vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D b = (surface[i] + dir * (j*division + lengthBalcon) - vert * widthBalcon) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D c = (surface[i] + dir * (j*division + lengthBalcon) - vert * (widthBalcon-WIDTH_BORDER_BALCON) ) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	vector3D d = (surface[i] + dir * (j*division) - vert * (widthBalcon-WIDTH_BORDER_BALCON)) + vector3D(0,0,elevation+WIDTH_BORDER_BALCON);
	m.addBoxFull(a, b, c, d, HEIGHT_BALCON - WIDTH_BORDER_BALCON);
      }
    }
  }
}

void Floor::addWindows(Mesh& m, int elevation) const{
  //Création des fenêtres en fct de la longueur de chaque façade
  // faire varier la distance/largeur des fenetres avec le type
  
  for(int i = 0 ; i<surface.size();i++){
    double length = Norm(surface[i+1==surface.size()?0:i+1]-surface[i]);
    double widthWindow = WIDTH_WINDOW*(1+type/10);
    int divs = length/widthWindow;
    double division = length / divs;
    vector3D dir = Normalized(surface[i+1==surface.size()?0:i+1]-surface[i]);
    vector3D vert = vector3D(-1 * dir[1],dir[0],dir[2]);
    //placer une fenetre toutes les 3 divisions
    for(int j = 2 ; j<divs; j+=5){
      {//bas
	vector3D a = (surface[i] + dir * (j*division) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation);
	vector3D b = (surface[i] + dir * (j*division + widthWindow) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation);
	vector3D c = (surface[i] + dir * (j*division + widthWindow) - vert * WIDTH_BORDER_WINDOW ) + vector3D(0,0,1+elevation);
	vector3D d = (surface[i] + dir * (j*division) - vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation);
	m.addBoxFull(a, b, c, d, WIDTH_BORDER_WINDOW);
      }
      {//haut
	vector3D a = (surface[i] + dir * (j*division) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+HEIGHT_WINDOW);
	vector3D b = (surface[i] + dir * (j*division + widthWindow) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+HEIGHT_WINDOW);
	vector3D c = (surface[i] + dir * (j*division + widthWindow) - vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+HEIGHT_WINDOW);
	vector3D d = (surface[i] + dir * (j*division) - vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+HEIGHT_WINDOW);
	m.addBoxFull(a, b, c, d, WIDTH_BORDER_WINDOW);
      }
      {//côté 1
	vector3D a = (surface[i] + dir * (j*division) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	vector3D b = (surface[i] + dir * (j*division + WIDTH_BORDER_WINDOW) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	vector3D c = (surface[i] + dir * (j*division + WIDTH_BORDER_WINDOW) - vert * WIDTH_BORDER_WINDOW ) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	vector3D d = (surface[i] + dir * (j*division) - vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	m.addBoxFull(a, b, c, d, HEIGHT_WINDOW-WIDTH_BORDER_WINDOW);
      }
      {//côté 2
	vector3D a = (surface[i] + dir * (j*division+ widthWindow - WIDTH_BORDER_WINDOW) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	vector3D b = (surface[i] + dir * (j*division+ widthWindow) + vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	vector3D c = (surface[i] + dir * (j*division+ widthWindow) - vert * WIDTH_BORDER_WINDOW ) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	vector3D d = (surface[i] + dir * (j*division+ widthWindow - WIDTH_BORDER_WINDOW ) - vert * WIDTH_BORDER_WINDOW) + vector3D(0,0,1+elevation+WIDTH_BORDER_WINDOW);
	m.addBoxFull(a, b, c, d, HEIGHT_WINDOW-WIDTH_BORDER_WINDOW);
      }
    }
  }

}
