#include "mesh.h"

void Mesh::write(std::string const & filePath) const {
    std::cout << "Writing mesh to a .obj file, please wait..." << std::endl;
    std::ofstream file(filePath.data(), std::ofstream::out);
    file.exceptions(std::ifstream::failbit | std::ifstream::badbit);

    file << "mtllib test.mtl" << std::endl;
    file << "o City" << std::endl;
    for (long j = 0; j < points.size(); j++) {
        file << "v " << points[j][0] << " " << points[j][1] << " " << points[j][2] << std::endl;
    }
    file << "usemtl default" << std::endl;
    //    file << "s off" <<std::endl;
    for (long j = 0; j < indices.size(); j++) {
        if (indices[j].getColor() != "default") {
            file << "usemtl " << indices[j].getColor() << std::endl;
            file << "f " << (indices[j][0] + 1) << " " << (indices[j][1] + 1) << " " << (indices[j][2] + 1) << std::endl;
            file << "usemtl default" << std::endl;
        } else {
            file << "f " << (indices[j][0] + 1) << " " << (indices[j][1] + 1) << " " << (indices[j][2] + 1) << std::endl;
        }
    }
    file.close();
    std::cout << "Obj file written !" << std::endl;
}

//créer 2 triangles

void Mesh::addQuad(const vector3D& a, const vector3D& b, const vector3D& c, const vector3D& d, std::string color) {
    points.push_back(a);
    points.push_back(b);
    points.push_back(c);
    points.push_back(d);

    int size = points.size() - 1;
    indices.push_back(IndexMesh(size - 3, size - 2, size - 1, color));
    indices.push_back(IndexMesh(size - 3, size - 1, size, color));
}

void Mesh::addTriangle(const vector3D& a, const vector3D& b, const vector3D& c, std::string color) {
    points.push_back(a);
    points.push_back(b);
    points.push_back(c);

    int size = points.size() - 1;
    indices.push_back(IndexMesh(size - 2, size - 1, size, color));
}

/**
 * Boite simple
 */
void Mesh::addBox(const vector3D& a, const vector3D& b) {
    vector3D a1(a[0], a[1], a[2]);
    vector3D a2(a[0], b[1], a[2]);
    vector3D a3(a[0], b[1], b[2]);
    vector3D a4(a[0], a[1], b[2]);
    vector3D b1(b[0], a[1], a[2]);
    vector3D b2(b[0], b[1], a[2]);
    vector3D b3(b[0], b[1], b[2]);
    vector3D b4(b[0], a[1], b[2]);

    points.push_back(a1); //7
    points.push_back(a2); //6
    points.push_back(a3); //5
    points.push_back(a4); //4
    points.push_back(b1); //3
    points.push_back(b2); //2
    points.push_back(b3); //1
    points.push_back(b4); //0

    int size = points.size() - 1;
    //soit faire des addQuad soit gerer les triangles
    indices.push_back(IndexMesh(size - 7, size - 6, size - 5));
    indices.push_back(IndexMesh(size - 7, size - 5, size - 4));

    indices.push_back(IndexMesh(size - 6, size, size - 3));
    indices.push_back(IndexMesh(size - 6, size - 3, size - 5));

    indices.push_back(IndexMesh(size, size - 2, size - 1));
    indices.push_back(IndexMesh(size, size - 3, size - 2));

    indices.push_back(IndexMesh(size - 1, size - 4, size - 7));
    indices.push_back(IndexMesh(size - 1, size - 7, size - 2));

    //dessous
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );

    //dessus
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );
}

/**
 * Boite 
 */
void Mesh::addBox(const vector3D& a, const vector3D& b, const vector3D& c, const vector3D& d, double h, std::string color) {
    vector3D a2(a[0], a[1], a[2] + h);
    vector3D b2(b[0], b[1], b[2] + h);
    vector3D c2(c[0], c[1], c[2] + h);
    vector3D d2(d[0], d[1], d[2] + h);

    points.push_back(a); //7
    points.push_back(b); //6
    points.push_back(c); //5
    points.push_back(d); //4
    points.push_back(a2); //3
    points.push_back(b2); //2
    points.push_back(c2); //1
    points.push_back(d2); //0

    int size = points.size() - 1;

    //soit faire des addQuad soit gerer les triangles
    indices.push_back(IndexMesh(size - 7, size - 6, size - 2)); //a,b,b2
    indices.push_back(IndexMesh(size - 7, size - 2, size - 3)); //a,b2,a2

    indices.push_back(IndexMesh(size - 6, size - 5, size - 1)); //b,c,c2
    indices.push_back(IndexMesh(size - 6, size - 1, size - 2)); //b,c2,b2

    indices.push_back(IndexMesh(size - 5, size - 4, size)); //c,d,d2
    indices.push_back(IndexMesh(size - 5, size, size - 1)); //c,d2,c2

    indices.push_back(IndexMesh(size - 4, size - 7, size - 3)); //d,a,a2
    indices.push_back(IndexMesh(size - 4, size - 3, size)); //d,a2,d2

    //dessous
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );

    //dessus
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );
    //i.push_back( Vector(size - 7, size - 5, size - 4 ) );
}

void Mesh::addBoxFull(const vector3D& a, const vector3D& b, const vector3D& c, const vector3D& d, double h, std::string color) {
    vector3D a2(a[0], a[1], a[2] + h);
    vector3D b2(b[0], b[1], b[2] + h);
    vector3D c2(c[0], c[1], c[2] + h);
    vector3D d2(d[0], d[1], d[2] + h);

    points.push_back(a); //7
    points.push_back(b); //6
    points.push_back(c); //5
    points.push_back(d); //4
    points.push_back(a2); //3
    points.push_back(b2); //2
    points.push_back(c2); //1
    points.push_back(d2); //0

    int size = points.size() - 1;

    //soit faire des addQuad soit gerer les triangles
    indices.push_back(IndexMesh(size - 7, size - 6, size - 2)); //a,b,b2
    indices.push_back(IndexMesh(size - 7, size - 2, size - 3)); //a,b2,a2

    indices.push_back(IndexMesh(size - 6, size - 5, size - 1)); //b,c,c2
    indices.push_back(IndexMesh(size - 6, size - 1, size - 2)); //b,c2,b2

    indices.push_back(IndexMesh(size - 5, size - 4, size)); //c,d,d2
    indices.push_back(IndexMesh(size - 5, size, size - 1)); //c,d2,c2

    indices.push_back(IndexMesh(size - 4, size - 7, size - 3)); //d,a,a2
    indices.push_back(IndexMesh(size - 4, size - 3, size)); //d,a2,d2

    //dessus
    indices.push_back(IndexMesh(size - 3, size - 2, size - 1)); //a2,b2,c2
    indices.push_back(IndexMesh(size - 3, size - 1, size)); //a2,c2,d2

    //dessous
    indices.push_back(IndexMesh(size - 7, size - 6, size - 5)); //a,b,c
    indices.push_back(IndexMesh(size - 7, size - 5, size - 4)); //a,c,d
}

/**
 * Boite 
 */
void Mesh::addBox3(const vector3D& a, const vector3D& b, const vector3D& c, double h, std::string color) {
    vector3D a2(a[0], a[1], a[2] + h);
    vector3D b2(b[0], b[1], b[2] + h);
    vector3D c2(c[0], c[1], c[2] + h);

    points.push_back(a); //5
    points.push_back(b); //4
    points.push_back(c); //3
    points.push_back(a2); //2
    points.push_back(b2); //1
    points.push_back(c2); //0


    int size = points.size() - 1;

    //soit faire des addQuad soit gerer les triangles
    indices.push_back(IndexMesh(size - 5, size - 4, size - 1)); //a,b,b2
    indices.push_back(IndexMesh(size - 5, size - 1, size - 2)); //a,b2,a2

    indices.push_back(IndexMesh(size - 4, size - 3, size)); //b,c,c2
    indices.push_back(IndexMesh(size - 4, size, size - 1)); //b,c2,b2

    indices.push_back(IndexMesh(size - 3, size - 5, size - 2)); //c,a,a2
    indices.push_back(IndexMesh(size - 3, size - 2, size)); //c,a2,c2
}

void Mesh::addPyramid(const vector3D& a, const vector3D& b,const vector3D& c, const vector3D& d, const vector3D& e, std::string color) {
  /*    vector3D d(a[0], b[1], b[2]);
	vector3D e(b[0], a[1], b[2]);*/

    points.push_back(a); //4
    points.push_back(b); //3
    points.push_back(c); //2
    points.push_back(d); //1
    points.push_back(e); //0

    int size = points.size() - 1;

    indices.push_back(IndexMesh(size , size - 4, size - 3));
    indices.push_back(IndexMesh(size , size - 3, size - 2));
    indices.push_back(IndexMesh(size , size - 2, size - 1));
    indices.push_back(IndexMesh(size , size - 1, size - 4));
}

void Mesh::addCylinder(const vector3D& a, const vector3D& b, double r, int sectors, std::string color) {
  
    double angle = 360/sectors;
    for(int i = 0; i<sectors; i++){      
      points.push_back(vector3D(a[0] + r * cos(angle*i*M_PI/180), a[1] + r * sin(angle*i*M_PI/180), a[2]));  
      points.push_back(vector3D(b[0] + r * cos(angle*i*M_PI/180), b[1] + r * sin(angle*i*M_PI/180), b[2]));     
    }
    long size = points.size() - 1;
    points.push_back(b); //1
    points.push_back(a); //0    
    for(int i = 0; i<sectors; i++){
      if(i<sectors-1){
	indices.push_back(IndexMesh(size-i*2, size-i*2-1, size -i*2-3, color));
	indices.push_back(IndexMesh(size-i*2, size-i*2-3, size-i*2-2, color));
	indices.push_back(IndexMesh(size+1, size-i*2, size - i*2-2, color));
	indices.push_back(IndexMesh(size+2, size-i*2-1, size -i*2-3, color));
      }else{
	indices.push_back(IndexMesh(size-i*2, size-i*2-1, size -1, color));
	indices.push_back(IndexMesh(size-i*2, size-1, size, color));
	indices.push_back(IndexMesh(size+1, size-i*2, size, color));
	indices.push_back(IndexMesh(size+2, size-i*2-1, size -1, color));
      }
    }
}

void Mesh::addCone(const vector3D& a, const vector3D& b, double r, int sectors, std::string color) {
    double angle = 360/sectors;
    for(int i = 0; i<sectors; i++){      
      points.push_back(vector3D(a[0] + r * cos(angle*i*M_PI/180), a[1] + r * sin(angle*i*M_PI/180), a[2]));  
    }
    long size = points.size() - 1;
    points.push_back(b); //1
    points.push_back(a); //0
    for(int i = 0; i<sectors; i++){
      if(i<sectors-1){
	indices.push_back(IndexMesh(size-i, size-i-1, size +1, color));
	indices.push_back(IndexMesh(size-i, size-i-1, size +2, color));
      }else{
	indices.push_back(IndexMesh(size-i, size, size +1, color));
	indices.push_back(IndexMesh(size-i, size, size +2, color));
      }
    }
}

void Mesh::addSphere(const vector3D& center, double radius, int rings, int sectors, std::string color) {
    float const R = 1. / (float) (rings - 1);
    float const S = 1. / (float) (sectors - 1);
    int r, s;

    /*
    u.resize(rings * sectors * 3);
    i.resize(rings * sectors );
    //  normals.resize(rings * sectors * 3);
    //texcoords.resize(rings * sectors * 2);
    std::vector<Vector3D>::iterator v = u.begin();
    std::vector<Vector3D>::iterator w = i.begin();
    //std::vector<double>::iterator n = normals.begin();
    //std::vector<double>::iterator t = texcoords.begin();*/
    for (r = 0; r < rings; r++) {
        for (s = 0; s < sectors; s++) {
            float const y = sin(-M_PI_2 + M_PI * r * R);
            float const x = cos(2 * M_PI * s * S) * sin(M_PI * r * R);
            float const z = sin(2 * M_PI * s * S) * sin(M_PI * r * R);

            vector3D p(x*radius, y*radius, z * radius);
            points.push_back(p);
            /*      *t++ = s*S;
             *t++ = r*R;

             *v++ = x * radius;
             *v++ = y * radius;
             *v++ = z * radius;

             *n++ = x;
             *n++ = y;
             *n++ = z;*/
        }
    }

    /*  indices.resize(rings * sectors * 4);
        std::vector<int>::iterator i = indices.begin();*/
    for (r = 0; r < rings - 1; r++) {
        for (s = 0; s < sectors - 1; s++) {

            /*      *i++ = r * sectors + s;
             *i++ = r * sectors + (s+1);
             *i++ = (r+1) * sectors + (s+1);
             *i++ = (r+1) * sectors + s;*/
        }
    }
}

std::ostream & operator <<(std::ostream & os, const Mesh & mesh) {
  os << "Number of points: " <<mesh.points.size()<< std::endl;
  /*    for (std::vector<vector3D>::const_iterator it = mesh.points.begin(); it != mesh.points.end(); it++) {
        os << *it << std::endl;
	}*/
  os << "Number of triangles : " << mesh.indices.size() << std::endl;
  /*for (std::vector<indexMesh>::const_iterator it = mesh.indices.begin(); it != mesh.indices.end(); it++) {
    os << *it << std::endl;
    }*/
  return os;
}
