#include "quadrangle.h"

Quadrangle::Quadrangle() {
}

Quadrangle::~Quadrangle() {
}

Quadrangle::Quadrangle(const Quadrangle& q) : points(q.points) {

}

Quadrangle::Quadrangle(const vector3D& v1, const vector3D& v2, const vector3D& v3, const vector3D& v4) : points {
    v1, v2, v3, v4
}
{

}

Quadrangle::Quadrangle(const vector3D& v1, const vector3D& v2) : points {
    v1, vector3D(v1[0], v2[1], v1[2]), v2, vector3D(v2[0], v1[1], v2[2])
}
{

}

void Quadrangle::grammaire(std::vector<Polygone> & list) const {
    double surface = area();
    if (stopDivision(QUADRANGLE_SURF_MIN, QUADRANGLE_SURF_MAX, surface)) { //Stopper la division si surface trop petite ou si probabilite d'arret
        list.push_back(Polygone(*this));
    } else {
        int randGrammar = rand() % 100;
        double dist01 = Norm(points[0] - points[1]);
        double dist12 = Norm(points[1] - points[2]);
        double dist23 = Norm(points[2] - points[3]);
        double dist30 = Norm(points[3] - points[0]);
        double distMin = min(dist01, min(dist12, min(dist23, dist30))) * 2; //Eviter qu'un triangle soit trop applatit (un cote trop petit))
        if (((distMin >= TRIANGLE_EDGE_MIN) && (randGrammar < Q_QQ)) || ((distMin < TRIANGLE_EDGE_MIN) && (randGrammar < Q_QQ_NO_TT))) { //Q -> QQ division
            //Couper en partant du côté le plus grand, jusqu'au cote oppose a celui-ci
            int maxEdgeRank = getMaxEdgeRank(dist01, dist12, dist23, dist30);
            double edgeCutRatio = getEdgeCutRatio();
            if ((maxEdgeRank == 0) || (maxEdgeRank == 2)) {
                Quadrangle(points[0],
                        points[0] + (points[1] - points[0]) * edgeCutRatio,
                        points[3] + (points[2] - points[3]) * (1 - edgeCutRatio),
                        points[3]).grammaire(list);
                Quadrangle(points[0] + (points[1] - points[0]) * edgeCutRatio,
                        points[1],
                        points[2],
                        points[3] + (points[2] - points[3]) * (1 - edgeCutRatio)).grammaire(list);
            } else { //1 ou 3
                Quadrangle(points[0],
                        points[1],
                        points[1] + (points[2] - points[1]) * edgeCutRatio,
                        points[0] + (points[3] - points[0]) * (1 - edgeCutRatio)).grammaire(list);
                Quadrangle(points[0] + (points[3] - points[0]) * (1 - edgeCutRatio),
                        points[1] + (points[2] - points[1]) * edgeCutRatio,
                        points[2],
                        points[3]).grammaire(list);
            }
        } else if (((distMin >= TRIANGLE_EDGE_MIN) && (randGrammar < Q_QT)) || ((distMin < TRIANGLE_EDGE_MIN) && (randGrammar < Q_QT_NO_TT))) { //Q -> QT division
            //Couper du côté le plus grand au point opposé le plus eloigné
            int maxEdgeRank = getMaxEdgeRank(dist01, dist12, dist23, dist30);
            vector3D edgeCutPoint = points[maxEdgeRank] + (points[(maxEdgeRank + 1) % 4] - points[maxEdgeRank]) * 0.5; //Point de coupe sur le plus grand cote
            int pointToCut = Norm(edgeCutPoint - points[(maxEdgeRank + 2) % 4]) < Norm(edgeCutPoint - points[(maxEdgeRank + 3) % 4]) ? (maxEdgeRank + 3) % 4 : (maxEdgeRank + 2) % 4; //Num du point oppose le plus eloigne
            double edgeCutRatio = getEdgeCutRatio();
            Quadrangle(points[(maxEdgeRank + 2) % 4],
                    points[(maxEdgeRank + 3) % 4],
                    (((maxEdgeRank + pointToCut) % 2) == 0) ? points[maxEdgeRank] : points[maxEdgeRank] + (points[(maxEdgeRank + 1) % 4] - points[maxEdgeRank]) * edgeCutRatio,
                    (((maxEdgeRank + pointToCut) % 2) != 0) ? points[(maxEdgeRank + 1) % 4] : points[maxEdgeRank] + (points[(maxEdgeRank + 1) % 4] - points[maxEdgeRank]) * edgeCutRatio).grammaire(list);
            Triangle(points[maxEdgeRank] + (points[(maxEdgeRank + 1) % 4] - points[maxEdgeRank]) * edgeCutRatio,
                    points[((((maxEdgeRank + pointToCut) % 2) == 0) ? maxEdgeRank + 1 : maxEdgeRank + 3) % 4],
                    points[(((maxEdgeRank + pointToCut) % 2) == 0) ? (maxEdgeRank + 2) % 4 : maxEdgeRank]).grammaire(list);
        } else if (distMin >= TRIANGLE_EDGE_MIN) { //Q -> TT division
            //Toujours couper en prenant la diagonale la plus petite
            if (Norm(points[0] - points[2]) < Norm(points[1] - points[3])) {
                Triangle(points[0], points[1], points[2]).grammaire(list);
                Triangle(points[0], points[2], points[3]).grammaire(list);
            } else {
                Triangle(points[0], points[1], points[3]).grammaire(list);
                Triangle(points[1], points[2], points[3]).grammaire(list);
            }
        }
    }
}

std::ostream & operator <<(std::ostream & os, const Quadrangle & quadrangle) {
    return os << "quad(" << quadrangle.points[0] << ", " << quadrangle.points[1] << ", " << quadrangle.points[2] << ", " << quadrangle.points[3] << ")";
}
