#include "roof.h"

Roof::Roof() : surface(), type(0), level(1) {
    
}

Roof::Roof(const Polygone& surface, int type, int level) : surface(surface), type(type), level(level) {

}

void Roof::createMesh(Mesh &m) const {
  vector3D a(surface[0][0], surface[0][1], surface[0][2] + MIN_HEIGHT_FLOOR * (level - 1) + MIN_HEIGHT_GROUNDFLOOR);
  vector3D b(surface[1][0], surface[1][1], surface[1][2] + MIN_HEIGHT_FLOOR * (level - 1) + MIN_HEIGHT_GROUNDFLOOR);
  vector3D c(surface[2][0], surface[2][1], surface[2][2] + MIN_HEIGHT_FLOOR * (level - 1) + MIN_HEIGHT_GROUNDFLOOR);
  if (surface.getPoints().size() == 3) {
    m.addTriangle(a, b, c);
  } else if (surface.getPoints().size() == 4) {
    vector3D d(surface[3][0], surface[3][1], surface[3][2] + MIN_HEIGHT_FLOOR * (level - 1) + MIN_HEIGHT_GROUNDFLOOR);
    m.addQuad(a, b, c, d);

    if(surface.area()>(MIN_WIDTH_BUILDING*2)*(MIN_WIDTH_BUILDING*2)){
      //si nombre d'étage élevé toit plat + 30% cylindre pour réserve d'eau
      if(level>=MID_COUNT_LEVEL){
	m.addQuad(a, b, c, d);
	if((int) rand()%100 >70){
	  line diag1(a, c);
	  line diag2(b, d);
	  vector3D e = diag1.intersect(diag2);
	  m.addCylinder(e, vector3D(e[0], e[1], e[2] + 4),  2);
	}
      }else{
	//si l'angle du batiment est différent de 90 +/- 10 %=> toit plat
	vector3D u(b[0] - a[0], b[1] - a[1], 0);
	vector3D v(d[0] - a[0], d[1] - a[1], 0);
	double angle = (180 / M_PI) * acos((u * v) / (Norm(u) * Norm(v)));
	//sinon 50% toit plat/toit en prisme
	if((angle<80 or angle >100)  or (int) rand()%100 >50){
	  m.addQuad(a, b, c, d);
	  if((int) rand()%100 >70){
	    line diag1(a, c);
	    line diag2(b, d);
	    vector3D e = diag1.intersect(diag2);
	    m.addCylinder(e, vector3D(e[0], e[1], e[2] + 4),  2);
	  }
	}else{
	  //TODO : prendre le côté le plus long ou plus court
	  double height = Norm(b-a)/3;
	  vector3D e= vector3D((a+b)/2)+vector3D(0,0,height);
	  vector3D f= vector3D((c+d)/2)+vector3D(0,0,height);
	  m.addQuad(a, e, f, d,"toit");
	  m.addQuad(e, b, c, f,"toit");
	  m.addTriangle(a, b, e);
	  m.addTriangle(c, d, f);
	}
      }
    }else{
      //si l'angle du batiment est différent de 90 +/- 10 %=> toit plat
      vector3D u(b[0] - a[0], b[1] - a[1], 0);
      vector3D v(d[0] - a[0], d[1] - a[1], 0);
      double angle = (180 / M_PI) * acos((u * v) / (Norm(u) * Norm(v)));
      if(angle<80 or angle >100){
	m.addQuad(a, b, c, d);
      }else{

	//si nombre étage élevé toit en pyramide à 25%
	//si nombre étage moyen 50% toit semi plat/toit prisme
	//si nombre étages bas toit prisme
	if(level>=HIGH_COUNT_LEVEL){
	  if((int) rand()%100 >75){
	    line diag1(a, c);
	    line diag2(b, d);
	    vector3D e = diag1.intersect(diag2) +vector3D(0,0, Norm(c-a)/3);
	    m.addPyramid(a, b, c, d, e);
	  }else m.addQuad(a, b, c, d);
	}else if(level>=MID_COUNT_LEVEL){
	  m.addQuad(a, b, c, d);
	}else{
	  double height = Norm(b-a)/4;
	  vector3D e= vector3D((a+b)/2)+vector3D(0,0,height);
	  vector3D f= vector3D((c+d)/2)+vector3D(0,0,height);
	  m.addQuad(a, e, f, d,"toit");
	  m.addQuad(e, b, c, f,"toit");
	  m.addTriangle(a, b, e);
	  m.addTriangle(c, d, f);
	}
      }      
    }


  }
}

std::ostream & operator <<(std::ostream & os, const Roof & roof) {
  return os << "roof(surface: " << roof.surface << ", type: " << roof.type << ", level: " << roof.level << ")" << std::endl;
}
